"""
These tests expect to be with 4 ranks
"""
import faulthandler
faulthandler.enable()

import torch
import pytest
import numpy as np

import dace
from dace.sdfg import utils as sdfg_utils

from daceml.util import utils
from daceml.distributed import schedule
from daceml.distributed.utils import find_map_containing, compile_and_call, arange_with_size


def test_elementwise_1d(sizes, sdfg_name):
    @dace
    def programtry2(x: dace.int64[64], y: dace.int64[64]):
        y[:] = x + 5
    sdfg = programtry2.to_sdfg()
    sdfg.name = sdfg_name

    for k, v in sdfg.arrays.items():
        if not v.transient and type(v) == dace.data.Array:
            v.storage = dace.dtypes.StorageType.GPU_Global


    sdfg.apply_gpu_transformations()
    map_entry = find_map_containing(sdfg, "")
    schedule.lower(sdfg, {map_entry: sizes})

    X = torch.from_numpy(arange_with_size([64])).cuda()
    expected = X + 5

    from mpi4py import MPI
    commworld = MPI.COMM_WORLD
    commworld.Barrier()
    rank = commworld.Get_rank()
    size = commworld.Get_size()

    num_required_ranks = 1
    if size < num_required_ranks:
        raise ValueError(
            "This test requires at least {} ranks".format(num_required_ranks))


    func = sdfg_utils.distributed_compile(sdfg, commworld)
    assert_args = dict(atol=1e-5, rtol=1e-5)

    result = torch.empty_like(X)
    func(x=X, y=result)
    if rank == 0:
        np.testing.assert_allclose(result.cpu().numpy(),
                                   expected.cpu().numpy(),
                                   err_msg="Rank {} failed".format(rank),
                                   **assert_args)

    print(f"Passed with GPU {sizes}!")


test_elementwise_1d([2], "test_elementwise2")
